import {
  DatabaseErrorException,
  NotFoundException,
  RpcExceptionCode,
  RpcValidationPipe,
} from '@mobicoop/ddd-library';
import { Controller, UsePipes } from '@nestjs/common';
import { CommandBus } from '@nestjs/cqrs';
import { GrpcMethod, RpcException } from '@nestjs/microservices';
import { DeleteAuthenticationCommand } from '@modules/authentication/core/application/commands/delete-authentication/delete-authentication.command';
import { DeleteAuthenticationRequestDto } from './dtos/delete-authentication.request.dto';

@UsePipes(
  new RpcValidationPipe({
    whitelist: true,
    forbidUnknownValues: false,
  }),
)
@Controller()
export class DeleteAuthenticationGrpcController {
  constructor(private readonly commandBus: CommandBus) {}

  @GrpcMethod('AuthenticationService', 'Delete')
  async delete(data: DeleteAuthenticationRequestDto): Promise<void> {
    try {
      await this.commandBus.execute(new DeleteAuthenticationCommand(data));
    } catch (error: any) {
      if (error instanceof NotFoundException)
        throw new RpcException({
          code: RpcExceptionCode.NOT_FOUND,
          message: error.message,
        });

      if (error instanceof DatabaseErrorException)
        throw new RpcException({
          code: RpcExceptionCode.INTERNAL,
          message: error.message,
        });
      throw new RpcException({
        code: RpcExceptionCode.UNKNOWN,
        message: error.message,
      });
    }
  }
}
