import { Test, TestingModule } from '@nestjs/testing';
import { ConfigService } from '@nestjs/config';
import { HttpService } from '@nestjs/axios';
import { OpaDecisionMaker } from '@modules/authorization/infrastructure/opa.decision-maker';
import { of } from 'rxjs';
import {
  Action,
  Domain,
} from '@modules/authorization/core/domain/authorization.types';

const mockConfigService = {
  get: jest.fn().mockImplementation(() => 'http://localhost:8181/v1/data/'),
};
const mockHttpService = {
  post: jest
    .fn()
    .mockImplementationOnce(() => {
      return of({
        status: 200,
        data: {
          decision_id: 'b22965f0-f48a-4fcf-84db-b6f31cd07b8c',
          result: {
            allow: true,
          },
        },
      });
    })
    .mockImplementationOnce(() => {
      return of({
        status: 200,
        data: {
          decision_id: '5a648ea2-6790-4337-b63c-2ebdf225466f',
          result: {
            allow: false,
          },
        },
      });
    })
    .mockImplementationOnce(() => {
      throw new Error();
    }),
};

describe('OPA decision maker', () => {
  let opaDecisonMaker: OpaDecisionMaker;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        {
          provide: ConfigService,
          useValue: mockConfigService,
        },
        {
          provide: HttpService,
          useValue: mockHttpService,
        },
        OpaDecisionMaker,
      ],
    }).compile();

    opaDecisonMaker = module.get<OpaDecisionMaker>(OpaDecisionMaker);
  });

  afterEach(async () => {
    jest.clearAllMocks();
  });

  it('should be defined', () => {
    expect(opaDecisonMaker).toBeDefined();
  });
  it('should return a positive decision', async () => {
    const decision: boolean = await opaDecisonMaker.decide(
      Domain.USER,
      Action.READ,
      [
        {
          name: 'owner',
          value: '96d99d44-e0a6-458e-a656-de2a400d60a8',
        },
        {
          name: 'id',
          value: '96d99d44-e0a6-458e-a656-de2a400d60a8',
        },
      ],
    );
    expect(decision).toBeTruthy();
  });
  it('should return a negative decision', async () => {
    const decision: boolean = await opaDecisonMaker.decide(
      Domain.USER,
      Action.READ,
      [
        {
          name: 'owner',
          value: '96d99d44-e0a6-458e-a656-de2a400d60a8',
        },
        {
          name: 'id',
          value: '96d99d44-e0a6-458e-a656-de2a400d60a9',
        },
      ],
    );
    expect(decision).toBeFalsy();
  });
  it('should return a negative decision if an error occurs', async () => {
    const decision: boolean = await opaDecisonMaker.decide(
      Domain.USER,
      Action.READ,
      [
        {
          name: 'owner',
          value: '96d99d44-e0a6-458e-a656-de2a400d60a8',
        },
        {
          name: 'id',
          value: '96d99d44-e0a6-458e-a656-de2a400d60a9',
        },
      ],
    );
    expect(decision).toBeFalsy();
  });
});
